using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ReadMoreAnalogInputs
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the board
        /// </summary>
        private PCI3001_NET.PCI3001 board;

        /// <summary>
        /// Number of acquisition
        /// </summary>
        private int acqCount;

        /// <summary>
        /// Delegate
        /// </summary>
        /// <param name="args"></param>
        private delegate void IrqGuiActionDelegate(PCI3001_NET.EndOfSequenceEventArgs args);

        /// <summary>
        /// Constructor of the GUI
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
            this.board = null;
        }

        /// <summary>
        /// Event handler: occurs when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            int count = 0;
            string[] src = null;

            // Parameters
            this.convertingTimeTextBox.Text = Properties.Settings.Default.convertingTime;

            // Get the number of boards available
            try
            {
                count = PCI3001_NET.PCI3001.GetBoardNumber();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error while getting the number of APCI-3001: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                this.Close();
                return;
            }

            // Some boards available?
            if (count <= 0)
            {
                MessageBox.Show("No APCI-3001 available...",
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                this.Close();
                return;
            }

            // Create the source
            src = new string[count];
            for (int i = 0; i < count; i++)
            {
                src[i] = i.ToString();
            }
            boardSelectionComboBox.DataSource = src;
            boardSelectionComboBox.Invalidate();

            // Create the channel list view
            this.UpdateChannelListView();
        }

        /// <summary>
        /// Event handler: form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.convertingTime = convertingTimeTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Update the channel list view
        /// </summary>
        private void UpdateChannelListView()
        {
            int count = 0;
            this.channelsListView.Items.Clear();
            if (singleEndedRadioButton.Checked == true)
                count = 16;
            else
                count = 8;

            // Create and add the items
            for (int i = 0; i < count; i++)
            {
                ListViewItem item = new ListViewItem(new string[] { String.Empty, i.ToString(), "Waiting data..." });
                item.Checked = true;
                this.channelsListView.Items.Add(item);
            }
        }

        /// <summary>
        /// Click on the "Select" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectButton_Click(object sender, EventArgs e)
        {
            int retval = 0;

            // Board was already selected
            if (this.board != null)
            {
                retval = this.board.ResetBoardIntRoutine();
                if (retval != 0)
                {
                    MessageBox.Show("Error while reseting the interrupt routine: " + retval.ToString(),
                        "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                    return;
                }
            }

            try
            {
                this.board = PCI3001_NET.PCI3001.OpenBoard(boardSelectionComboBox.SelectedIndex);
                if (this.board == null)
                    throw new Exception("Error opening the board.");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error while opening the board: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Install an interrupt routine
            retval = this.board.SetBoardIntRoutine();
            if (retval != 0)
            {
                MessageBox.Show("Error while installing the interrupt routine: " + retval.ToString(),
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // End of sequence
            this.board.EndOfSequenceEventReceived += new PCI3001_NET.PCI3001.EndOfSequenceEventHandler(board_EndOfSequenceEventReceived);
            this.readMoreAnalogInputsGroupBox.Enabled = true;
        }

        /// <summary>
        /// Action of the IRQ on the GUI
        /// </summary>
        /// <param name="args"></param>
        void IrqGuiAction(PCI3001_NET.EndOfSequenceEventArgs args)
        {
            if (InvokeRequired)
                this.Invoke(new IrqGuiActionDelegate(IrqGuiAction), args);
            else
            {
                List<int> index = this.GetSelectedIndexes();
                for (int i = 0; i < index.Count; i++)
                {
                    channelsListView.Items[index[i]].SubItems[2].Text = args.Args[i].ToString();
                }

                this.progressToolStripProgressBar.Visible = false;
                this.statusToolStripStatusLabel.Visible = false;
                this.boardSelectionGroupBox.Enabled = true;
                this.readMoreAnalogInputsGroupBox.Enabled = true;
            }
        }

        /// <summary>
        /// End of sequence received (read more analog inputs)
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_EndOfSequenceEventReceived(object source, PCI3001_NET.EndOfSequenceEventArgs args)
        {
            this.IrqGuiAction(args);
        }

        /// <summary>
        /// Change the input mode
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void singleEndedRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            UpdateChannelListView();
        }

        /// <summary>
        /// Get the selected indexes
        /// </summary>
        /// <returns></returns>
        private List<int> GetSelectedIndexes()
        {
            List<int> ret = new List<int>();

            for (int i = 0; i < channelsListView.Items.Count; i++)
            {
                if (channelsListView.Items[i].Checked)
                    ret.Add(i);
            }

            return ret;
        }

        /// <summary>
        /// Event handler: click on the "Start button"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            ushort convertingTime = 0;
            try
            {
                convertingTime = Convert.ToUInt16(convertingTimeTextBox.Text);
                if ((convertingTime < 10) || (convertingTime > 32767))
                    throw new Exception("Converting time must be between 10 and 32767");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Wrong converting time: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            this.singleEndedRadioButton.Enabled = false;
            this.differentialRadioButton.Enabled = false;
            this.startButton.Enabled = false;
            this.stopButton.Enabled = true;
            this.boardSelectionGroupBox.Enabled = false;
            this.progressToolStripProgressBar.Visible = true;
            this.statusToolStripStatusLabel.Text = "Waiting data...";
            this.statusToolStripStatusLabel.Visible = true;
            this.acquisitionModeGroupBox.Enabled = false;
            this.acqCount = 0;
            this.readBackgroundWorker.RunWorkerAsync(new ReadWorkerParameters(this.board,
                this.GetSelectedIndexes(),
                convertingTime,
                singleEndedRadioButton.Checked ? 1 : 2));
        }

        /// <summary>
        /// Event handler: click on the "Stop button"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.stopButton.Enabled = false;
            this.readBackgroundWorker.CancelAsync();
        }

        /// <summary>
        /// Job of the background worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            BackgroundWorker worker = (BackgroundWorker)sender;
            ReadWorkerParameters p = (ReadWorkerParameters)e.Argument;
            byte[] channels = new byte[p.index.Count];
            byte[] gain = new byte[p.index.Count];
            byte[] polarity = new byte[p.index.Count];

            // Create
            for (int i = 0; i < p.index.Count; i++)
            {
                channels[i] = (byte)p.index[i];
                gain[i] = 0; // Gain x1
                polarity[i] = 0; // Bipolard
            }

            // Set input mode
            retval = p.board.SetAnalogInputMode((byte)p.inputMode);
            if (retval != 0)
            {
                MessageBox.Show("Error while setting the analog input mode: " + retval.ToString(),
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Endless loop
            for (; ; )
            {
                ushort[] values = new ushort[p.index.Count];

                if (worker.CancellationPending)
                    return;

                // Read
                retval = p.board.ReadMoreAnalogInputs((byte)p.index.Count,
                    channels,
                    gain,
                    polarity,
                    p.convertingTime,
                    0,
                    values);
                if (retval != 0)
                {
                    MessageBox.Show("Error while reading the analog inputs: " + retval.ToString(),
                        "Error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                    return;
                }

                // Report progress
                worker.ReportProgress(1, new ReadWorkerValues(values, p.index));

                System.Threading.Thread.Sleep(50);
            }
        }

        /// <summary>
        /// Worker completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.boardSelectionGroupBox.Enabled = true;
            this.startButton.Enabled = true;
            this.progressToolStripProgressBar.Visible = false;
            this.statusToolStripStatusLabel.Visible = false;
            this.singleEndedRadioButton.Enabled = true;
            this.differentialRadioButton.Enabled = true;
            this.acquisitionModeGroupBox.Enabled = true;
        }

        /// <summary>
        /// Worker report some progress
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ReadWorkerValues v = (ReadWorkerValues)e.UserState;
            for (int i = 0; i < v.index.Count; i++)
            {
                this.channelsListView.Items[v.index[i]].SubItems[2].Text = v.values[i].ToString();
            }

            this.acqCount++;
            this.statusToolStripStatusLabel.Text = this.acqCount.ToString() + " data acquired.";
        }

        /// <summary>
        /// Select/Deselect interrupt mode
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void noRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (noRadioButton.Checked == true)
            {
                this.startButton.Enabled = true;
                this.readButton.Enabled = false;
            }
            else
            {
                this.startButton.Enabled = false;
                this.readButton.Enabled = true;
            }
        }

        /// <summary>
        /// Click on read
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readButton_Click(object sender, EventArgs e)
        {
            ushort convertingTime = 0;
            try
            {
                convertingTime = Convert.ToUInt16(convertingTimeTextBox.Text);
                if ((convertingTime < 10) || (convertingTime > 32767))
                    throw new Exception("Converting time must be between 10 and 32767");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Wrong converting time: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            this.progressToolStripProgressBar.Visible = true;
            this.statusToolStripStatusLabel.Text = "Waiting interrupt...";
            this.statusToolStripStatusLabel.Visible = true;
            this.boardSelectionGroupBox.Enabled = false;
            this.readMoreAnalogInputsGroupBox.Enabled = false;

            List<int> index = this.GetSelectedIndexes();
            byte[] channels = new byte[index.Count];
            byte[] gain = new byte[index.Count];
            byte[] polarity = new byte[index.Count];
            int retval = 0;
            ushort[] values = new ushort[index.Count];

            // Create
            for (int i = 0; i < index.Count; i++)
            {
                channels[i] = (byte)index[i];
                gain[i] = 0; // Gain x1
                polarity[i] = 0; // Bipolar
            }

            // Set input mode
            retval = board.SetAnalogInputMode((byte)(singleEndedRadioButton.Checked ? 1 : 2));
            if (retval != 0)
            {
                MessageBox.Show("Error while setting the analog input mode: " + retval.ToString(),
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Read
            retval = board.ReadMoreAnalogInputs((byte)index.Count,
                channels,
                gain,
                polarity,
                convertingTime,
                1,
                values);
            if (retval != 0)
            {
                MessageBox.Show("Error while reading the analog inputs: " + retval.ToString(),
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }
        }
    }
}